@@ -1,8 +1,6 @@
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
-from typing import Optional
-
 import attr
 
 
@@ -14,10 +12,7 @@ class DataSource:
         name (str): Name for the Data Source. Used in sanity metric
             column names.
         from_expr (str): FROM expression - often just a fully-qualified
-            table name. Sometimes a subquery. May contain the string
-            ``{dataset}`` which will be replaced with an app-specific
-            dataset for Glean apps. If the expression is templated
-            on dataset, default_dataset is mandatory.
+            table name. Sometimes a subquery.
         experiments_column_type (str or None): Info about the schema
             of the table or view:
 
@@ -37,19 +32,14 @@ class DataSource:
         submission_date_column (str, optional): Name of the column
             that contains the submission date (as a date, not
             timestamp). Defaults to 'submission_date'.
-        default_dataset (str, optional): The value to use for
-            `{dataset}` in from_expr if a value is not provided
-            at runtime. Mandatory if from_expr contains a
-            `{dataset}` parameter.
     """
     name = attr.ib(validator=attr.validators.instance_of(str))
-    _from_expr = attr.ib(validator=attr.validators.instance_of(str))
+    from_expr = attr.ib(validator=attr.validators.instance_of(str))
     experiments_column_type = attr.ib(default='simple', type=str)
     client_id_column = attr.ib(default='client_id', type=str)
     submission_date_column = attr.ib(default='submission_date', type=str)
-    default_dataset = attr.ib(default=None, type=Optional[str])
 
-    EXPERIMENT_COLUMN_TYPES = (None, "simple", "native", "glean")
+    EXPERIMENT_COLUMN_TYPES = (None, "simple", "native")
 
     @experiments_column_type.validator
     def _check_experiments_column_type(self, attribute, value):
@@ -59,28 +49,6 @@ class DataSource:
                 f"{repr(self.EXPERIMENT_COLUMN_TYPES)}"
             )
 
-    @default_dataset.validator
-    def _check_default_dataset_provided_if_needed(self, attribute, value):
-        self.from_expr_for(None)
-
-    def from_expr_for(self, dataset: Optional[str]) -> str:
-        """Expands the ``from_expr`` template for the given dataset.
-        If ``from_expr`` is not a template, returns ``from_expr``.
-
-        Args:
-            dataset (str or None): Dataset name to substitute
-                into the from expression.
-        """
-        effective_dataset = dataset or self.default_dataset
-        if effective_dataset is None:
-            try:
-                return self._from_expr.format()
-            except Exception as e:
-                raise ValueError(
-                    f"{self.name}: from_expr contains a dataset template but no value was provided."  # noqa:E501
-                ) from e
-        return self._from_expr.format(dataset=effective_dataset)
-
     @property
     def experiments_column_expr(self):
         if self.experiments_column_type is None:
@@ -89,7 +57,7 @@ class DataSource:
         elif self.experiments_column_type == 'simple':
             return """AND (
                     ds.{submission_date} != e.enrollment_date
-                    OR `mozfun.map.get_key`(
+                    OR `moz-fx-data-shared-prod.udf.get_key`(
                         ds.experiments, '{experiment_slug}'
                     ) IS NOT NULL
                 )"""
@@ -97,29 +65,18 @@ class DataSource:
         elif self.experiments_column_type == 'native':
             return """AND (
                     ds.{submission_date} != e.enrollment_date
-                    OR `mozfun.map.get_key`(
+                    OR `moz-fx-data-shared-prod.udf.get_key`(
                         ds.experiments, '{experiment_slug}'
                     ).branch IS NOT NULL
             )"""
 
         elif self.experiments_column_type == 'glean':
-            return """AND (
-                    ds.{submission_date} != e.enrollment_date
-                    OR `mozfun.map.get_key`(
-                        ds.ping_info.experiments, '{experiment_slug}'
-                    ).branch IS NOT NULL
-                )"""
+            raise NotImplementedError
 
         else:
             raise ValueError
 
-    def build_query(
-        self,
-        metric_list,
-        time_limits,
-        experiment_slug,
-        from_expr_dataset=None,
-    ):
+    def build_query(self, metric_list, time_limits, experiment_slug):
         """Return a nearly-self contained SQL query.
 
         This query does not define ``enrollments`` but otherwise could
@@ -141,7 +98,7 @@ class DataSource:
         GROUP BY e.client_id, e.analysis_window_start, e.analysis_window_end""".format(
             client_id=self.client_id_column,
             submission_date=self.submission_date_column,
-            from_expr=self.from_expr_for(from_expr_dataset),
+            from_expr=self.from_expr,
             fddr=time_limits.first_date_data_required,
             lddr=time_limits.last_date_data_required,
             metrics=',\n            '.join(
@@ -165,14 +122,14 @@ class DataSource:
                 Metric(
                     name=self.name + '_has_contradictory_branch',
                     data_source=self,
-                    select_expr=agg_any("""`mozfun.map.get_key`(
+                    select_expr=agg_any("""`moz-fx-data-shared-prod.udf.get_key`(
                 ds.experiments, '{experiment_slug}'
             ) != e.branch"""),
                 ),
                 Metric(
                     name=self.name + '_has_non_enrolled_data',
                     data_source=self,
-                    select_expr=agg_any("""`mozfun.map.get_key`(
+                    select_expr=agg_any("""`moz-fx-data-shared-prod.udf.get_key`(
                 ds.experiments, '{experiment_slug}'
             ) IS NULL""".format(experiment_slug=experiment_slug))
                 ),
@@ -183,36 +140,21 @@ class DataSource:
                 Metric(
                     name=self.name + '_has_contradictory_branch',
                     data_source=self,
-                    select_expr=agg_any("""`mozfun.map.get_key`(
+                    select_expr=agg_any("""`moz-fx-data-shared-prod.udf.get_key`(
                 ds.experiments, '{experiment_slug}'
             ).branch != e.branch"""),
                 ),
                 Metric(
                     name=self.name + '_has_non_enrolled_data',
                     data_source=self,
-                    select_expr=agg_any("""`mozfun.map.get_key`(
+                    select_expr=agg_any("""`moz-fx-data-shared-prod.udf.get_key`(
                 ds.experiments, '{experiment_slug}'
             ).branch IS NULL""".format(experiment_slug=experiment_slug))
                 ),
             ]
 
         elif self.experiments_column_type == 'glean':
-            return [
-                Metric(
-                    name=self.name + '_has_contradictory_branch',
-                    data_source=self,
-                    select_expr=agg_any("""`mozfun.map.get_key`(
-                ds.ping_info.experiments, '{experiment_slug}'
-            ).branch != e.branch"""),
-                ),
-                Metric(
-                    name=self.name + '_has_non_enrolled_data',
-                    data_source=self,
-                    select_expr=agg_any("""`mozfun.map.get_key`(
-                ds.ping_info.experiments, '{experiment_slug}'
-            ).branch IS NULL""".format(experiment_slug=experiment_slug))
-                ),
-            ]
+            raise NotImplementedError
 
         else:
             raise ValueError
@@ -223,24 +165,18 @@ class Metric:
     """Represents an experiment metric.
 
     Needs to be combined with an analysis window to be measurable!
-
-    Args:
-        name (str): A slug; uniquely identifies this metric in tables
-        data_source (DataSource): where to find the metric
-        select_expr (str): a SQL snippet representing a clause of a SELECT
-            expression describing how to compute the metric; must include an
-            aggregation function since it will be GROUPed BY client_id
-            and branch
-        friendly_name (str): A human-readable dashboard title for this metric
-        description (str): A paragraph of Markdown-formatted text describing
-            what the metric measures, to be shown on dashboards
     """
     name = attr.ib(type=str)
     data_source = attr.ib(type=DataSource)
     select_expr = attr.ib(type=str)
-    friendly_name = attr.ib(type=Optional[str], default=None)
-    description = attr.ib(type=Optional[str], default=None)
-    bigger_is_better = attr.ib(type=bool, default=True)
+
+    def from_data_source(self, data_source: DataSource) -> "Metric":
+        """Returns an instance of the Metric drawing from a different DataSource.
+
+        This is particularly useful for Glean, where pings from e.g. Fenix
+        and Fenix Nightly are sent to different tables with identical schemas.
+        """
+        return attr.evolve(self, data_source=data_source)
 
 
 def agg_sum(select_expr):
